// Jim Bumgardner 11/25/2015

// To convert frames to gif using image magick: convert -delay 5 -loop 0 frame*.png -layers Optimize output.gif

float orbitRad = 5;
float tileIncr = orbitRad;
float dotRad = orbitRad*0.4; // width of dot using ratio of orbit
float heightRatio = sqrt(3)/2; //  hegagonal y/x aspect ratio (height of equilateral triangle)

int gw = int(540 / tileIncr) + 2;
int gh = int(gw/heightRatio);
boolean saveFrames = false; // turn this on to save frames for gif
boolean drawBG= false;
PImage myPic;
int maxFrames = 50; // length of cycle
String picName = "mona540.png";

float getPhase(float px, float py)
{
  float phase;
  int ix = int(px);
  int iy = int(py);
  ix = max(0,min(width-1,ix));
  iy = max(0,min(height-1,iy));
  color clr = myPic.pixels[iy*myPic.width+ix];
  return (clr & 0x0FF)*PI*0.5/255.0;
}

color getColor(float px, float py)
{
  float phase;
  int ix = int(px);
  int iy = int(py);
  ix = max(0,min(width-1,ix));
  iy = max(0,min(height-1,iy));
  return myPic.pixels[iy*myPic.width+ix];
}

class Orbit {
  float x1,y1,x2,y2;
  float p1,p2,p3;  // phases
  color c1,c2,c3;  // colors
  Orbit(float x1,float y1, float x2,float y2) {
    this.x1 = x1;
    this.y1 = y1;
    this.x2 = x2;
    this.y2 = y2;
    this.p1 = getPhase(x1,y1);
    this.p2 = getPhase(x2,y2);
    this.c1 = getColor(x1,y1);
    this.c2 = getColor(x2,y2);
  }
  
  void draw(float r) {
    float px,py,ang;
    ang = map(r,0,1,0,TWO_PI);
    px = map(r,0,1,x1,x2);
    py = map(r,0,1,y1,y2);
    if (drawBG) {
       noFill();
       stroke(128);
       ellipse(px, py, orbitRad, orbitRad); 
    }
    pushMatrix();
    translate(px,py);
      float phase;
      color clr;
      phase = map(r,0,1,this.p1,this.p2);
      clr = lerpColor(this.c1,this.c2,r);
      noStroke();
      fill(clr);
      float px2 = orbitRad*cos(ang+phase);
      float py2 = orbitRad*sin(ang+phase);
      ellipse(px2,py2,dotRad,dotRad);
    popMatrix();
  }
};

ArrayList<Orbit> orbits;

void setup()
{
  size(540,540);
  ellipseMode(RADIUS);
  frameRate(20);
  myPic = loadImage(picName);
  myPic.loadPixels();
  orbits = new ArrayList<Orbit>();

  float tileIncrX = tileIncr;
  float tileIncrY = tileIncr*heightRatio;

  for (int y = 0; y < gh+2; ++y) {
    for (int x = 0; x < gw+2; ++x) {
       float px1 = x*tileIncrX + (y % 2 == 0? tileIncrX*0.5 : 0) - orbitRad;
       float py1 = y*tileIncrY - orbitRad;
       float px2 = px1 + tileIncrX/2;
       float py2 = py1 + tileIncrY;
       Orbit orb = new Orbit(px1,py1,px2,py2);
       orbits.add(orb);
    }
  }
}

void draw() {
  background(0);

  float phase = frameCount/float(maxFrames);
  phase -= int(phase);

  for (Orbit orb : orbits) {
    orb.draw(phase);
  }

  if (saveFrames) {
    if (frameCount <= maxFrames) {
      saveFrame("frame_####.png");
    }
    else {
      noLoop();
    }
  }
}